// -*- C++ -*-
#include "AGILe/Charybdis/CharybdisFHerwig.hh"
#include "AGILe/Charybdis/CharybdisWrapper.hh"
#include "AGILe/FHerwig/FHerwig.hh"
#include "AGILe/FHerwig/FHerwigWrapper65.hh"
#include "AGILe/Utils.hh"
#include "HepMC/HEPEVT_Wrapper.h"


//-----------------------------------------------------------------------------
// Implementation file for class : CharybdisFHerwig
// This is the file-reading interface wrapper only. It contains an instance of
// FHerwig which does most of the work. This class is only to provide access to
// the parameter settings.
//
// Authors: Doug Gingrich
//          Andy Buckley
//-----------------------------------------------------------------------------


namespace AGILe {

  // Standard constructor
  CharybdisFHerwig::CharybdisFHerwig() {
    // Set flag to read from LHA event file
    FC_HWPROC.IPROC = -100;
    // PDF defaults
    FC_HEPRUP.PDFGUP[0] = -1;
    FC_HEPRUP.PDFGUP[1] = -1;
    FC_HEPRUP.PDFSUP[0] = -1;
    FC_HEPRUP.PDFSUP[1] = -1;
  }


  // Set string parameters
  bool CharybdisFHerwig::setParam(const string& name, const string& value) {
    Generator::setParam(name, value);

    // Set double-valued params
    if (name == "MPLNCK") {
      getLog() << Log::INFO << "Setting ... (MPLNCK) = " << value << endl;
      FC_BHPARM.MPLNCK = asDouble(value);
    } else if (name == "MINMSS") {
      getLog() << Log::INFO << "Setting ... (MINMSS) = " << value << endl;
      FC_BHPARM.MINMSS = asDouble(value);
    } else if (name == "MAXMSS") {
      getLog() << Log::INFO << "Setting ... (MAXMSS) = " << value << endl;
      FC_BHPARM.MAXMSS = asDouble(value);
    } else if (name == "THWMAX") {
      getLog() << Log::INFO << "Setting ... (THWMAX) = " << value << endl;
      FC_BH1003.THWMAX = asDouble(value);
    } else if (name == "RMMINM") {
      getLog() << Log::INFO << "Setting ... (RMMINM) = " << value << endl;
      FC_BH1003.RMMINM = asDouble(value);

    // Set int-valued params
    } else if (name == "PDFGUP") {
      getLog() << Log::INFO << "Setting Charybdis PDFGUP = " << value << endl;
      FC_HEPRUP.PDFGUP[0] = asInt(value);
      FC_HEPRUP.PDFGUP[1] = asInt(value);
    } else if (name == "PDFSUP") {
      getLog() << Log::INFO << "Setting Charybdis = " << value << endl;
      FC_HEPRUP.PDFSUP[0] = asInt(value);
      FC_HEPRUP.PDFSUP[1] = asInt(value);
    } else if (name == "MSSDEF") {
      getLog() << Log::INFO << "Setting ... (MSSDEF) = " << value << endl;
      FC_BHPARM.MSSDEF = asInt(value);
    } else if (name == "NBODY") {
      getLog() << Log::INFO << "Setting ... (NBODY) = " << value << endl;
      FC_BHPARM.NBODY = asInt(value);
    } else if (name == "IBHPRN") {
      getLog() << Log::INFO << "Setting ... (IBHPRN) = " << value << endl;
      FC_BHPARM.IBHPRN = asInt(value);
    } else if (name == "MSSDEC") {
      getLog() << Log::INFO << "Setting ... (MSSDEC) = " << value << endl;
      FC_BHPARM.MSSDEC = asInt(value);
    } else if (name == "TOTDIM") {
      getLog() << Log::INFO << "Setting ... (TOTDIM) = " << value << endl;
      FC_BLACKH.TOTDIM = asInt(value);

    // Set bool-valued params
    } else if (name == "TIMVAR") {
      getLog() << Log::INFO << "Setting ... (TIMVAR) = " << value << endl;
      FC_BHPARM.TIMVAR = asInt(value);
    } else if (name == "GTSCA") {
      getLog() << Log::INFO << "Setting ... (GTSCA) = " << value << endl;
      FC_BHPARM.GTSCA = asInt(value);
    } else if (name == "GRYBDY") {
      getLog() << Log::INFO << "Setting ... (GRYBDY) = " << value << endl;
      FC_BHPARM.GRYBDY = asInt(value);
    } else if (name == "KINCUT") {
      getLog() << Log::INFO << "Setting ... (KINCUT) = " << value << endl;
      FC_BHPARM.KINCUT = asInt(value);
    } else if (name == "YRCSEC") {
      getLog() << Log::INFO << "Setting ... (YRCSEC) = " << value << endl;
      FC_BH1003.YRCSEC = asInt(value);
    } else if (name == "RMBOIL") {
      getLog() << Log::INFO << "Setting ... (RMBOIL) = " << value << endl;
      FC_BH1003.RMBOIL = asInt(value);
    } else if (FHerwig::setParam(name, value) != SUCCESS) {
      getLog() << Log::ERROR << "CharybdisFHerwig doesn't have a parameter called " << name << endl;
      return FAILURE;
    }
    return SUCCESS;
  }

}


// Class factory
extern "C" {
  AGILe::Generator* create() { return new AGILe::CharybdisFHerwig(); }
  void destroy(AGILe::Generator* gen) { delete gen; }
}
